package cz.muni.pa165.generated.server;

import com.fasterxml.jackson.databind.ObjectMapper;
import cz.muni.pa165.generated.server.model.Cat;
import cz.muni.pa165.generated.server.model.HelloResponse;
import org.junit.jupiter.api.Test;
import org.openapitools.jackson.nullable.JsonNullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.math.BigDecimal;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

/**
 * Integration tests. Run by "maven verify".
 */
@SpringBootTest
@AutoConfigureMockMvc
public class IT {

    private static final Logger log = LoggerFactory.getLogger(IT.class);

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    ObjectMapper objectMapper;

    @Test
    void testHelloWorld() throws Exception {
        log.debug("testHelloWorld() running");
        String expectedResponse = "Hello World!";
        String response = mockMvc.perform(get("/api/hello"))
                                 .andExpect(status().isOk())
                                 .andExpect(jsonPath("$.message").value(expectedResponse))
                                 .andReturn()
                                 .getResponse()
                                 .getContentAsString();
        log.debug("response: {}", response);
        HelloResponse helloResponse = objectMapper.readValue(response, HelloResponse.class);
        assertThat("response", helloResponse.getMessage(), is(equalTo(expectedResponse)));
    }

    @Test
    void testCreateCat() throws Exception {
        log.debug("testCreateCat() running");
        String species = "Lynx lynx";
        Integer weight = 10;
        BigDecimal age = BigDecimal.valueOf(8);
        Boolean vaccinated = true;

        String response = mockMvc.perform(
                                         post("/cats?species=" + species + "&weight=" + weight + "&age=" + age +
                                                      "&vaccinated=" + vaccinated))
                                 .andExpect(status().isOk())
                                 .andExpect(jsonPath("$.species").value(species))
                                 .andExpect(jsonPath("$.weight").value(weight))
                                 .andExpect(jsonPath("$.age").value(age))
                                 .andExpect(jsonPath("$.vaccinated").value(vaccinated))
                                 .andReturn()
                                 .getResponse()
                                 .getContentAsString();

        log.debug("response = {}", response);

        Cat cat = objectMapper.readValue(response, Cat.class);

        assertThat("species", cat.getSpecies(), is(equalTo(species)));
        assertThat("weight", cat.getWeight(), is(equalTo(JsonNullable.of(weight))));
        assertThat("age", cat.getAge(), is(equalTo(JsonNullable.of(age))));
        assertThat("vaccinated", cat.getVaccinated(), is(equalTo(vaccinated)));
    }


}
